/*
 * p_hello.c -- a hello program (in pthread)
 */
#include <stdio.h>
#include <sys/types.h>
#include <pthread.h>

#define MAX_THREAD 1000 /* Maximum number of threads allowed */

/* A struct to hold the parameters passed to each thread */
typedef struct {
	int id;
} parm;

/* The function that each thread will execute */
void *hello(void *arg)
{
	parm *p=(parm *)arg;
	printf("Hello from node %d\n", p->id);
	pthread_exit(NULL);
}

void main(int argc, char* argv[]) {
	int n,i;
	pthread_t *threads; /* the thread identifiers */
	pthread_attr_t attr; /* set of thread attributes */
	parm *p; /* parameters for each thread */

	if (argc != 2)
	{
		printf ("Usage: %s <no. of threads>\n",argv[0]);
		exit(1);
	}

	n=atoi(argv[1]);

	if ((n < 1) || (n > MAX_THREAD))
	{
		printf ("The no of threads should between 1 and %d.\n",MAX_THREAD);
		exit(1);
	}

	threads=(pthread_t *)malloc(n*sizeof(*threads));
	pthread_attr_init(&attr);

	p=(parm *)malloc(sizeof(parm)*n);
	/* Start up the thread(s) */

	for (i=0; i<n; i++)
	{
		p[i].id=i;
		pthread_create(&threads[i], &attr, hello, (void *)(p+i));
	}

	/* Synchronize the completion of each thread. */

       	for (i=0; i<n; i++)
	{
		pthread_join(threads[i],NULL);
	}
	free(p);
}
